#include <cerrno>
#include <msp/core/systemerror.h>
#include <msp/io/handle_private.h>
#include <msp/strings/format.h>
#include "datagramsocket.h"
#include "socket_private.h"

namespace Msp {
namespace Net {

DatagramSocket::DatagramSocket(Family f, int p):
	Socket(f, SOCK_DGRAM, p)
{
#ifdef WIN32
	WSAEventSelect(priv->handle, *priv->event, FD_READ|FD_CLOSE);
#endif
	set_events(IO::P_INPUT);
}

int DatagramSocket::connect(const SockAddr &addr)
{
	sockaddr_storage sa;
	socklen_t size = addr.fill_sockaddr(sa);

	int err = ::connect(priv->handle, reinterpret_cast<sockaddr *>(&sa), size);
	if(err==-1)
	{
#ifdef WIN32
		throw system_error("connect", WSAGetLastError());
#else
		throw system_error("connect");
#endif
	}

	delete peer_addr;
	peer_addr = addr.copy();

	delete local_addr;
	size = sizeof(sockaddr_storage);
	getsockname(priv->handle, reinterpret_cast<sockaddr *>(&sa), &size);
	local_addr = SockAddr::create(sa);

	connected = true;

	return (err==0)?0:1;
}

unsigned DatagramSocket::sendto(const char *buf, unsigned size, const SockAddr &addr_)
{
	if(size==0)
		return 0;

	sockaddr_storage addr;
	socklen_t addr_len = addr_.fill_sockaddr(addr);

	int ret = ::sendto(priv->handle, buf, size, 0, reinterpret_cast<sockaddr *>(&addr), addr_len);
	if(ret<0)
	{
		if(errno==EAGAIN)
			return 0;
		else
		{
#ifdef WIN32
			throw system_error("sendto", WSAGetLastError());
#else
			throw system_error("sendto");
#endif
		}
	}

	return ret;
}

unsigned DatagramSocket::recvfrom(char *buf, unsigned size, SockAddr *&addr_)
{
	if(size==0)
		return 0;

	sockaddr_storage addr;
	socklen_t addr_len = sizeof(sockaddr_storage);

	int ret = ::recvfrom(priv->handle, buf, size, 0, reinterpret_cast<sockaddr *>(&addr), &addr_len);
	if(ret<0)
	{
		if(errno==EAGAIN)
			return 0;
		else
		{
#ifdef WIN32
			throw system_error("recvfrom", WSAGetLastError());
#else
			throw system_error("recvfrom");
#endif
		}
	}

	addr_ = SockAddr::create(addr);

	return ret;
}

} // namespace Net
} // namespace Msp
